<?php
/**
 * @package     WordPress
 * @subpackage  Toggle - Collapse Section For Elementor
 * @author      support@themegum.com
 * @since       1.0.0
*/
defined('ABSPATH') or die();

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Box_Shadow;

class ToggleCollapse_Elementor_Widget extends Widget_Base {


  public function __construct( $data = [], $args = null ) {
    parent::__construct( $data, $args );

    $is_type_instance = $this->is_type_instance();

    if ( ! $is_type_instance && null === $args ) {
      throw new \Exception( '`$args` argument is required when initializing a full widget instance.' );
    }

    add_action( 'elementor/element/before_section_start', [ $this, 'enqueue_script' ] );

    if ( $is_type_instance ) {
      $this->_register_skins();

      $widget_name = $this->get_name();

      /**
       * Widget skin init.
       *
       * Fires when Elementor widget is being initialized.
       *
       * The dynamic portion of the hook name, `$widget_name`, refers to the widget name.
       *
       * @since 1.0.0
       *
       * @param Widget_Base $this The current widget.
       */
      do_action( "elementor/widget/{$widget_name}/skins_init", $this );
    }
  }

  /**
   * Get widget name.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return string Widget name.
   */
  public function get_name() {
    return 'temegum_toggle_collapse';
  }

  /**
   * Get widget title.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return string Widget title.
   */
  public function get_title() {

    return esc_html__( 'Toggle Collapse', 'toggle-collapse-elementor' );
  }

  /**
   * Get widget icon.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return string Widget icon.
   */
  public function get_icon() {
    return 'eicon-plus';
  }

  public function get_keywords() {
    return [ 'wordpress', 'widget', 'toggle','collapse' ];
  }

  /**
   * Get widget categories.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return array Widget categories.
   */
  public function get_categories() {
    return [ 'temegum' ];
  }

  protected function _register_controls() {



    $this->start_controls_section(
      'section_title',
      [
        'label' => esc_html__( 'Toggle', 'toggle-collapse-elementor' ),
      ]
    );

    $this->add_control(
      'toggle_text',
      [
        'label' => esc_html__( 'Toggle Text', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::TEXT,
        'label_block' => true,
        'dynamic' => [
          'active' => false,
        ],
        'default' => esc_html__( 'Text Here', 'toggle-collapse-elementor' ),
      ]
    );

    $this->add_responsive_control(
      'text_align',
      [
        'label' => esc_html__( 'Alignment', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::CHOOSE,
        'options' => [
          'left' => [
            'title' => esc_html__( 'Left', 'toggle-collapse-elementor' ),
            'icon' => 'eicon-text-align-left',
          ],
          'center' => [
            'title' => esc_html__( 'Center', 'toggle-collapse-elementor' ),
            'icon' => 'eicon-text-align-center',
          ],
          'right' => [
            'title' => esc_html__( 'Right', 'toggle-collapse-elementor' ),
            'icon' => 'eicon-text-align-right',
          ],
        ],
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link' => 'text-align: {{VALUE}};',
        ],
      ]
    );


    $this->add_control(
      'toggle_icon',
      [
        'label' => esc_html__( 'Toggle Icon', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SWITCHER,
        'label_on' => esc_html__( 'No', 'toggle-collapse-elementor' ),
        'label_off' => esc_html__( 'Yes', 'toggle-collapse-elementor' ),
        'separator' => 'before',
      ]
    );


    $this->add_control(
      'normal_icon',
      [
        'label' => esc_html__( 'Normal', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::ICONS,
        'exclude_inline_options'=>['svg'],
        'label_block' => false,
        'skin' => 'inline',
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_control(
      'collapse_icon',
      [
        'label' => esc_html__( 'Collapse', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::ICONS,
        'exclude_inline_options'=>['svg'],
        'label_block' => false,
        'skin' => 'inline',
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );

    $this->add_control(
      'icon_align',
      [
        'label' => esc_html__( 'Icon Position', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SELECT,
        'default' => 'left',
        'options' => [
          'left' => esc_html__( 'Before', 'toggle-collapse-elementor' ),
          'right' => esc_html__( 'After', 'toggle-collapse-elementor' ),
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'toggle_settings',
      [
        'label' => esc_html__( 'Section Settings', 'toggle-collapse-elementor' ),
      ]
    );    


    $this->add_control(
      'section_id',
      [
        'label' => esc_html__( 'Section CSS ID', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::TEXT,
        'dynamic' => [
          'active' => false,
        ],
        'default' => '',
        'title' => esc_html__( 'CSS ID from target section NOT this widget ID. You can founded on target section settings: Advanced > CSS ID', 'toggle-collapse-elementor' ),
        'description' => esc_html__( 'Please make sure the ID is unique and not used elsewhere on the page this form is displayed. This field allows A-z 0-9 & underscore chars without spaces. Separated by comma for multiIDs.', 'toggle-collapse-elementor' ),
      ]
    );


    $this->add_control(
      'toggle_state',
      [
        'label' => esc_html__( 'Expanded First', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SWITCHER,
        'label_on' => esc_html__( 'Yes', 'toggle-collapse-elementor' ),
        'label_off' => esc_html__( 'No', 'toggle-collapse-elementor' ),
        'default' => '',
      ]
    );

    $this->end_controls_section();


/*
 * style params
 */

    $this->start_controls_section(
      'title_style',
      [
        'label' => esc_html__( 'Toggle Text', 'toggle-collapse-elementor' ),
        'tab'   => Controls_Manager::TAB_STYLE,
      ]
    );    


    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_title',
        'selector' => '{{WRAPPER}} .toggle-text',
      ]
    );



    $this->add_control(
      'title_color',
      [
        'label' => esc_html__( 'Normal Color', 'toggle-collapse-elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-text' => 'color: {{VALUE}};',
        ]
      ]
    );


    $this->add_control(
      'title_hovercolor',
      [
        'label' => esc_html__( 'Hover Color', 'toggle-collapse-elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link:hover .toggle-text' => 'color: {{VALUE}};',
        ],
      ]
    );



    $this->add_responsive_control(
      'icon_gap',
      [
        'label' => esc_html__( 'Spacing', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'min' => 0,
            'max' => 200,
          ],
        ],
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link.icon-left .toggle-text' => 'margin-left: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .toggle-collapse-link.icon-right .toggle-text + .toggle-icons' => 'margin-left: {{SIZE}}{{UNIT}};',
        ],
      ]
    );


    $this->add_control(
      'collapse_title_icon',
      [
        'label' => esc_html__( 'Collapse Icon', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::HEADING,
        'separator' => 'before',
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_responsive_control(
      'collapse_icon_size',
      [
        'label' => esc_html__( 'Size', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'min' => 0,
            'max' => 100,
          ],
        ],
        'default'=> ['size'=>15],
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link .toggle-icon.toggle-collapse' => 'font-size: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_responsive_control(
      'colapse_icon_rotate',
      [
        'label' => esc_html__( 'Rotate', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SLIDER,
        'size_units' => [ 'deg' ],
        'default' => [
          'size' => 0,
          'unit' => 'deg',
        ],
        'tablet_default' => [
          'unit' => 'deg',
        ],
        'mobile_default' => [
          'unit' => 'deg',
        ],
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link .toggle-icon.toggle-collapse i' => 'transform: rotate({{SIZE}}{{UNIT}});',
        ],
      ]
    );

    $this->add_control(
      'colapse_normal_color',
      [
        'label' => esc_html__( 'Normal Color', 'toggle-collapse-elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link .toggle-icon.toggle-collapse' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_control(
      'collapse_hover_color',
      [
        'label' => esc_html__( 'Hover Color', 'toggle-collapse-elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link:hover .toggle-icon.toggle-collapse' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_control(
      'noraml_title_icon',
      [
        'label' => esc_html__( 'Expand Icon', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::HEADING,
        'separator' => 'before',
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_responsive_control(
      'icon_size',
      [
        'label' => esc_html__( 'Size', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'min' => 0,
            'max' => 100,
          ],
        ],
        'default'=> ['size'=>15],
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link .toggle-icon.toggle-open' => 'font-size: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_responsive_control(
      'icon_rotate',
      [
        'label' => esc_html__( 'Rotate', 'toggle-collapse-elementor' ),
        'type' => Controls_Manager::SLIDER,
        'size_units' => [ 'deg' ],
        'default' => [
          'size' => 0,
          'unit' => 'deg',
        ],
        'tablet_default' => [
          'unit' => 'deg',
        ],
        'mobile_default' => [
          'unit' => 'deg',
        ],
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link .toggle-icon.toggle-open i' => 'transform: rotate({{SIZE}}{{UNIT}});',
        ],
      ]
    );

    $this->add_control(
      'normal_color',
      [
        'label' => esc_html__( 'Normal Color', 'toggle-collapse-elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link .toggle-icon.toggle-open' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->add_control(
      'normal_hover_color',
      [
        'label' => esc_html__( 'Hover Color', 'toggle-collapse-elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .toggle-collapse-link:hover .toggle-icon.toggle-open' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'toggle_icon' => 'yes',
        ],
      ]
    );


    $this->end_controls_section();


  }


  protected function render() {

    $settings = $this->get_settings_for_display();

    extract( $settings );

    $toggle_icon_html = '';

    if( $toggle_icon =='yes' && ! empty( $normal_icon['value'] )){

      if($collapse_icon['value']==''){
        $collapse_icon = $normal_icon;
      }

      $toggle_icon_html = '<span class="toggle-icons"><span class="toggle-icon toggle-open"><i class="'.esc_attr($normal_icon['value']).'"></i></span>';
      $toggle_icon_html .= '<span class="toggle-icon toggle-collapse"><i class="'.esc_attr($collapse_icon['value']).'"></i></span></span>';
    }

?>
    <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
      <div class="toggle-collapse-link icon-<?php esc_attr_e($icon_align);?>" data-target="<?php esc_attr_e($section_id);?>" data-target-state="<?php esc_attr_e($toggle_state);?>">
        <?php if($icon_align=='left'){?> <?php print "{$toggle_icon_html}"; }?>
        <span class="toggle-text"><?php esc_html_e($toggle_text); ?></span>
        <?php if($icon_align=='right'){?> <?php print "{$toggle_icon_html}"; }?>
      </div>
    </div>
<?php
  }


  protected function content_template() {
    ?>
    <#
    var normal_iconHTML = elementor.helpers.renderIcon( view, settings.normal_icon, { 'aria-hidden': true }, 'i' , 'object' ),
      collapse_iconHTML = elementor.helpers.renderIcon( view, settings.collapse_icon, { 'aria-hidden': true }, 'i' , 'object' );
    #>
    <div class="toggle-collapse-link icon-{{{settings.icon_align}}}" data-target="{{{settings.section_id}}}">
          <#  if ( settings.toggle_icon=='yes'){ #>
          <span class="toggle-icons" <#  if ( settings.icon_align=='right'){ #>style="display:none;"<# }#>>
          <# if (normal_iconHTML.rendered  ) { #>
          <span class="toggle-icon toggle-open">{{{ normal_iconHTML.value }}}</span>
          <# } #>
          <# if ( collapse_iconHTML.rendered  ) { #>
          <span class="toggle-icon toggle-collapse">{{{ collapse_iconHTML.value }}}</span>
          <# } #>
          </span>
          <# } #>
          <span class="toggle-text">{{{ settings.toggle_text }}}</span>
          <#  if ( settings.toggle_icon=='yes'){ #>
          <span class="toggle-icons" <#  if ( settings.icon_align=='left'){ #>style="display:none;"<# }#>>
          <# if (normal_iconHTML.rendered  ) { #>
          <span class="toggle-icon toggle-open">{{{ normal_iconHTML.value }}}</span>
          <# } #>
          <# if ( collapse_iconHTML.rendered  ) { #>
          <span class="toggle-icon toggle-collapse">{{{ collapse_iconHTML.value }}}</span>
          <# } #>
          </span>
          <# } #>

    </div>
    <?php
  }


  public function enqueue_script( ) {
    wp_enqueue_style( 'temegum-toggle-collapse',TGCL_ELEMENTOR_URL."css/style.css",array());
    wp_enqueue_script( 'temegum-toggle-collapse', TGCL_ELEMENTOR_URL . 'js/toggle-collapse.js', array('jquery'), '1.0', false );
  }


}


// Register widget
\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new ToggleCollapse_Elementor_Widget() );

?>